@@ -107,6 +107,8 @@
 void proto_reg_handoff_its(void);
 void proto_register_its(void);
 
+static dissector_handle_t its_handle;
+
 static expert_field ei_its_no_sub_dis = EI_INIT;
 
 // TAP
@@ -132,6 +134,8 @@ static int proto_its_mapem = -1;
 static int proto_its_spatemv1 = -1;
 static int proto_its_spatem = -1;
 static int proto_its_cpm = -1;
+static int proto_its_imzm = -1;
+static int proto_its_vam = -1;
 static int proto_addgrpc = -1;
 
 /*
@@ -192,6 +196,7 @@ static gint ett_camssp_flags = -1;
 static dissector_table_t its_version_subdissector_table;
 static dissector_table_t its_msgid_subdissector_table;
 static dissector_table_t regionid_subdissector_table;
+static dissector_table_t cpmcontainer_subdissector_table;
 static dissector_table_t cam_pt_activation_table;
 
 typedef struct its_private_data {
@@ -205,7 +210,7 @@ typedef struct its_pt_activation_data {
     tvbuff_t *data;
 } its_pt_activation_data_t;
 
-// Specidic dissector for content of open type for regional extensions
+// Specific dissector for content of open type for regional extensions
 static int dissect_regextval_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data _U_)
 {
     its_private_data_t *re = (its_private_data_t*)data;
@@ -215,6 +220,16 @@ static int dissect_regextval_pdu(tvbuff_t *tvb, packet_info *pinfo, proto_tree *
     return tvb_captured_length(tvb);
 }
 
+// Specific dissector for content of open type for regional extensions
+static int dissect_cpmcontainers_pdu(tvbuff_t* tvb, packet_info* pinfo, proto_tree* tree, void* data _U_)
+{
+    its_header_t* hdr = (its_header_t*)data;
+    // XXX What to do when region_id = noRegion? Test length is zero?
+    if (!dissector_try_uint_new(cpmcontainer_subdissector_table, hdr->CpmContainerId, tvb, pinfo, tree, FALSE, NULL))
+        call_data_dissector(tvb, pinfo, tree);
+    return tvb_captured_length(tvb);
+}
+
 static int dissect_denmssp_pdu(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree *tree, void *data _U_)
 {
     static int * const denmssp_flags[] = {
@@ -287,62 +302,36 @@ static int dissect_camssp_pdu(tvbuff_t *tvb, packet_info *pinfo _U_, proto_tree
 // Generated by asn2wrs
 #include "packet-its-hf.c"
 
-// CauseCode/SubCauseCode management
-static int hf_its_trafficConditionSubCauseCode = -1;
-static int hf_its_accidentSubCauseCode = -1;
-static int hf_its_roadworksSubCauseCode = -1;
-static int hf_its_adverseWeatherCondition_PrecipitationSubCauseCode = -1;
-static int hf_its_adverseWeatherCondition_VisibilitySubCauseCode = -1;
-static int hf_its_adverseWeatherCondition_AdhesionSubCauseCode = -1;
-static int hf_its_adverseWeatherCondition_ExtremeWeatherConditionSubCauseCode = -1;
-static int hf_its_hazardousLocation_AnimalOnTheRoadSubCauseCode = -1;
-static int hf_its_hazardousLocation_ObstacleOnTheRoadSubCauseCode = -1;
-static int hf_its_hazardousLocation_SurfaceConditionSubCauseCode = -1;
-static int hf_its_hazardousLocation_DangerousCurveSubCauseCode = -1;
-static int hf_its_humanPresenceOnTheRoadSubCauseCode = -1;
-static int hf_its_wrongWayDrivingSubCauseCode = -1;
-static int hf_its_rescueAndRecoveryWorkInProgressSubCauseCode = -1;
-static int hf_its_slowVehicleSubCauseCode = -1;
-static int hf_its_dangerousEndOfQueueSubCauseCode = -1;
-static int hf_its_vehicleBreakdownSubCauseCode = -1;
-static int hf_its_postCrashSubCauseCode = -1;
-static int hf_its_humanProblemSubCauseCode = -1;
-static int hf_its_stationaryVehicleSubCauseCode = -1;
-static int hf_its_emergencyVehicleApproachingSubCauseCode = -1;
-static int hf_its_collisionRiskSubCauseCode = -1;
-static int hf_its_signalViolationSubCauseCode = -1;
-static int hf_its_dangerousSituationSubCauseCode = -1;
-
 static gint ett_its = -1;
 
 #include "packet-its-ett.c"
 
 // Deal with cause/subcause code management
 struct { CauseCodeType_enum cause; int* hf; } cause_to_subcause[] = {
-    { trafficCondition, &hf_its_trafficConditionSubCauseCode },
-    { accident, &hf_its_accidentSubCauseCode },
-    { roadworks, &hf_its_roadworksSubCauseCode },
-    { adverseWeatherCondition_Precipitation, &hf_its_adverseWeatherCondition_PrecipitationSubCauseCode },
-    { adverseWeatherCondition_Visibility, &hf_its_adverseWeatherCondition_VisibilitySubCauseCode },
-    { adverseWeatherCondition_Adhesion, &hf_its_adverseWeatherCondition_AdhesionSubCauseCode },
-    { adverseWeatherCondition_ExtremeWeatherCondition, &hf_its_adverseWeatherCondition_ExtremeWeatherConditionSubCauseCode },
-    { hazardousLocation_AnimalOnTheRoad, &hf_its_hazardousLocation_AnimalOnTheRoadSubCauseCode },
-    { hazardousLocation_ObstacleOnTheRoad, &hf_its_hazardousLocation_ObstacleOnTheRoadSubCauseCode },
-    { hazardousLocation_SurfaceCondition, &hf_its_hazardousLocation_SurfaceConditionSubCauseCode },
-    { hazardousLocation_DangerousCurve, &hf_its_hazardousLocation_DangerousCurveSubCauseCode },
-    { humanPresenceOnTheRoad, &hf_its_humanPresenceOnTheRoadSubCauseCode },
-    { wrongWayDriving, &hf_its_wrongWayDrivingSubCauseCode },
-    { rescueAndRecoveryWorkInProgress, &hf_its_rescueAndRecoveryWorkInProgressSubCauseCode },
-    { slowVehicle, &hf_its_slowVehicleSubCauseCode },
-    { dangerousEndOfQueue, &hf_its_dangerousEndOfQueueSubCauseCode },
-    { vehicleBreakdown, &hf_its_vehicleBreakdownSubCauseCode },
-    { postCrash, &hf_its_postCrashSubCauseCode },
-    { humanProblem, &hf_its_humanProblemSubCauseCode },
-    { stationaryVehicle, &hf_its_stationaryVehicleSubCauseCode },
-    { emergencyVehicleApproaching, &hf_its_emergencyVehicleApproachingSubCauseCode },
-    { collisionRisk, &hf_its_collisionRiskSubCauseCode },
-    { signalViolation, &hf_its_signalViolationSubCauseCode },
-    { dangerousSituation, &hf_its_dangerousSituationSubCauseCode },
+    { trafficCondition, &hf_its_trafficCondition1 },
+    { accident, &hf_its_accident2 },
+    { roadworks, &hf_its_roadworks3 },
+    { adverseWeatherCondition_Precipitation, &hf_its_adverseWeatherCondition_Precipitation19 },
+    { adverseWeatherCondition_Visibility, &hf_its_adverseWeatherCondition_Visibility18 },
+    { adverseWeatherCondition_Adhesion, &hf_its_adverseWeatherCondition_Adhesion6 },
+    { adverseWeatherCondition_ExtremeWeatherCondition, &hf_its_adverseWeatherCondition_ExtremeWeatherCondition17 },
+    { hazardousLocation_AnimalOnTheRoad, &hf_its_hazardousLocation_AnimalOnTheRoad11 },
+    { hazardousLocation_ObstacleOnTheRoad, &hf_its_hazardousLocation_ObstacleOnTheRoad10 },
+    { hazardousLocation_SurfaceCondition, &hf_its_hazardousLocation_SurfaceCondition9 },
+    { hazardousLocation_DangerousCurve, &hf_its_hazardousLocation_DangerousCurve96 },
+    { humanPresenceOnTheRoad, &hf_its_humanPresenceOnTheRoad12 },
+    { wrongWayDriving, &hf_its_wrongWayDriving14 },
+    { rescueAndRecoveryWorkInProgress, &hf_its_rescueAndRecoveryWorkInProgress15 },
+    { slowVehicle, &hf_its_slowVehicle26 },
+    { dangerousEndOfQueue, &hf_its_dangerousEndOfQueue27 },
+    { vehicleBreakdown, &hf_its_vehicleBreakdown91 },
+    { postCrash, &hf_its_postCrash92 },
+    { humanProblem, &hf_its_humanProblem93 },
+    { stationaryVehicle, &hf_its_stationaryVehicle94 },
+    { emergencyVehicleApproaching, &hf_its_emergencyVehicleApproaching95 },
+    { collisionRisk, &hf_its_collisionRisk97 },
+    { signalViolation, &hf_its_signalViolation98 },
+    { dangerousSituation, &hf_its_dangerousSituation99 },
     { reserved, NULL },
 };
 
@@ -357,6 +346,21 @@ find_subcause_from_cause(CauseCodeType_enum cause)
     return cause_to_subcause[idx].hf?cause_to_subcause[idx].hf:&hf_its_subCauseCode;
 }
 
+static unsigned char ita2_ascii[32] = {
+    '\0', 'T', '\r', 'O', ' ', 'H', 'N', 'M', '\n', 'L', 'R', 'G', 'I', 'P', 'C', 'V',
+    'E', 'Z', 'D', 'B', 'S', 'Y', 'F', 'X', 'A', 'W', 'J', '\0', 'U', 'Q', 'K'
+};
+
+static void
+append_country_code_fmt(proto_item *item, tvbuff_t *val_tvb)
+{
+  guint16 v = tvb_get_guint16(val_tvb, 0, ENC_BIG_ENDIAN);
+  v >>= 6;  /* 10 bits */
+  guint16 v1 = (v >> 5) & 0x1F;
+  guint16 v2 = v & 0x1F;
+  proto_item_append_text(item, " - %c%c", ita2_ascii[v1], ita2_ascii[v2]);
+}
+
 #include "packet-its-fn.c"
 
 static void
@@ -735,73 +739,6 @@ dsrc_delta_time_fmt(gchar *s, guint32 v)
   }
 }
 
-static void
-cpm_general_confidence_fmt(gchar *s, guint32 v)
-{
-  if (v == 0) {
-    snprintf(s, ITEM_LABEL_LENGTH, "unknown (%u)", v);
-  } else if (v == 101) {
-    snprintf(s, ITEM_LABEL_LENGTH, "unavailable (%u)", v);
-  } else {
-    snprintf(s, ITEM_LABEL_LENGTH, "%u%% (%u)", v, v);
-  }
-}
-
-static void
-cpm_distance_value_fmt(gchar *s, guint32 v)
-{
-  gint32 sv = (gint32)v;
-  snprintf(s, ITEM_LABEL_LENGTH, "%.2fm (%d)", sv * 0.01, sv);
-}
-
-static void
-cpm_distance_confidence_fmt(gchar *s, guint32 v)
-{
-  if (v == 102) {
-    snprintf(s, ITEM_LABEL_LENGTH, "unavailable (%d)", v);
-  } else if (v == 101) {
-    snprintf(s, ITEM_LABEL_LENGTH, "outOfRange (%d)", v);
-  } else {
-    snprintf(s, ITEM_LABEL_LENGTH, "%.2fm (%d)", v * 0.01, v);
-  }
-}
-
-static void
-cpm_speed_value_ext_fmt(gchar *s, guint32 v)
-{
-  gint32 sv = (gint32)v;
-  if (sv == 0) {
-    snprintf(s, ITEM_LABEL_LENGTH, "standstill (%d)", sv);
-  } else if (sv == 16383) {
-    snprintf(s, ITEM_LABEL_LENGTH, "unavailable (%d)", sv);
-  } else {
-    double vms = sv * 0.01;
-    snprintf(s, ITEM_LABEL_LENGTH, "%.2fm/s = %.1fkm/h (%d)",
-            vms, vms * 3.6, sv);
-  }
-}
-
-static void
-cpm_cartesian_angle_value_fmt(gchar *s, guint32 v)
-{
-  if (v == 3601) {
-    snprintf(s, ITEM_LABEL_LENGTH, "unavailable (%d)", v);
-  } else {
-    snprintf(s, ITEM_LABEL_LENGTH, "%.1f° (%d)", v * 0.1, v);
-  }
-}
-
-static void
-cpm_angle_confidence_fmt(gchar *s, guint32 v)
-{
-  if (v == 127) {
-    snprintf(s, ITEM_LABEL_LENGTH, "unavailable (%d)", v);
-  } else if (v == 126) {
-    snprintf(s, ITEM_LABEL_LENGTH, "outOfRange (%d)", v);
-  } else {
-    snprintf(s, ITEM_LABEL_LENGTH, "%.1f° (%d)", v * 0.1, v);
-  }
-}
 
 static void
 cpm_object_dimension_value_fmt(gchar *s, guint32 v)
@@ -809,17 +746,17 @@ cpm_object_dimension_value_fmt(gchar *s, guint32 v)
   snprintf(s, ITEM_LABEL_LENGTH, "%.1fm (%d)", v * 0.1, v);
 }
 
-static void
-cpm_object_dimension_confidence_fmt(gchar *s, guint32 v)
-{
-  if (v == 102) {
-    snprintf(s, ITEM_LABEL_LENGTH, "unavailable (%d)", v);
-  } else if (v == 101) {
-    snprintf(s, ITEM_LABEL_LENGTH, "outOfRange (%d)", v);
-  } else {
-    snprintf(s, ITEM_LABEL_LENGTH, "%.2fm (%d)", v * 0.01, v);
-  }
-}
+//static void
+//cpm_object_dimension_confidence_fmt(gchar *s, guint32 v)
+//{
+//  if (v == 102) {
+//    snprintf(s, ITEM_LABEL_LENGTH, "unavailable (%d)", v);
+//  } else if (v == 101) {
+//    snprintf(s, ITEM_LABEL_LENGTH, "outOfRange (%d)", v);
+//  } else {
+//    snprintf(s, ITEM_LABEL_LENGTH, "%.2fm (%d)", v * 0.01, v);
+//  }
+//}
 
 static int
 dissect_its_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data)
@@ -840,7 +777,7 @@ dissect_its_PDU(tvbuff_t *tvb, packet_info *pinfo, proto_tree *tree, void *data)
 static void
 its_msgid_prompt(packet_info *pinfo, gchar *result)
 {
-    guint32 msgid = GPOINTER_TO_UINT(p_get_proto_data(pinfo->pool, pinfo, hf_its_messageID, pinfo->curr_layer_num));
+    guint32 msgid = GPOINTER_TO_UINT(p_get_proto_data(pinfo->pool, pinfo, hf_its_messageId, pinfo->curr_layer_num));
 
     snprintf(result, MAX_DECODE_AS_PROMPT_LEN, "MsgId (%s%u)", UTF8_RIGHTWARDS_ARROW, msgid);
 }
@@ -848,7 +785,7 @@ its_msgid_prompt(packet_info *pinfo, gchar *result)
 static gpointer
 its_msgid_value(packet_info *pinfo)
 {
-    return p_get_proto_data(pinfo->pool, pinfo, hf_its_messageID, pinfo->curr_layer_num);
+    return p_get_proto_data(pinfo->pool, pinfo, hf_its_messageId, pinfo->curr_layer_num);
 }
 
 // Registration of protocols
@@ -857,102 +794,6 @@ void proto_register_its(void)
     static hf_register_info hf_its[] = {
         #include "packet-its-hfarr.c"
 
-    { &hf_its_roadworksSubCauseCode,
-      { "roadworksSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_RoadworksSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_postCrashSubCauseCode,
-      { "postCrashSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_PostCrashSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_vehicleBreakdownSubCauseCode,
-      { "vehicleBreakdownSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_VehicleBreakdownSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_dangerousSituationSubCauseCode,
-      { "dangerousSituationSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_DangerousSituationSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_dangerousEndOfQueueSubCauseCode,
-      { "dangerousEndOfQueueSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_DangerousEndOfQueueSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_rescueAndRecoveryWorkInProgressSubCauseCode,
-      { "rescueAndRecoveryWorkInProgressSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_RescueAndRecoveryWorkInProgressSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_signalViolationSubCauseCode,
-      { "signalViolationSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_SignalViolationSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_collisionRiskSubCauseCode,
-      { "collisionRiskSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_CollisionRiskSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_hazardousLocation_AnimalOnTheRoadSubCauseCode,
-      { "hazardousLocation_AnimalOnTheRoadSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_HazardousLocation_AnimalOnTheRoadSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_hazardousLocation_ObstacleOnTheRoadSubCauseCode,
-      { "hazardousLocation_ObstacleOnTheRoadSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_HazardousLocation_ObstacleOnTheRoadSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_hazardousLocation_SurfaceConditionSubCauseCode,
-      { "hazardousLocation_SurfaceConditionSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_HazardousLocation_SurfaceConditionSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_hazardousLocation_DangerousCurveSubCauseCode,
-      { "hazardousLocation_DangerousCurveSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_HazardousLocation_DangerousCurveSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_emergencyVehicleApproachingSubCauseCode,
-      { "emergencyVehicleApproachingSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_EmergencyVehicleApproachingSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_humanProblemSubCauseCode,
-      { "humanProblemSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_HumanProblemSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_stationaryVehicleSubCauseCode,
-      { "stationaryVehicleSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_StationaryVehicleSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_slowVehicleSubCauseCode,
-      { "slowVehicleSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_SlowVehicleSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_adverseWeatherCondition_PrecipitationSubCauseCode,
-      { "adverseWeatherCondition_PrecipitationSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_AdverseWeatherCondition_PrecipitationSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_adverseWeatherCondition_VisibilitySubCauseCode,
-      { "adverseWeatherCondition_VisibilitySubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_AdverseWeatherCondition_VisibilitySubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_adverseWeatherCondition_AdhesionSubCauseCode,
-      { "adverseWeatherCondition_AdhesionSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_AdverseWeatherCondition_AdhesionSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_adverseWeatherCondition_ExtremeWeatherConditionSubCauseCode,
-      { "adverseWeatherCondition_ExtremeWeatherConditionSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_AdverseWeatherCondition_ExtremeWeatherConditionSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_wrongWayDrivingSubCauseCode,
-      { "wrongWayDrivingSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_WrongWayDrivingSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_humanPresenceOnTheRoadSubCauseCode,
-      { "humanPresenceOnTheRoadSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_HumanPresenceOnTheRoadSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_accidentSubCauseCode,
-      { "accidentSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_AccidentSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
-    { &hf_its_trafficConditionSubCauseCode,
-      { "trafficConditionSubCauseCode", "its.subCauseCode",
-        FT_UINT32, BASE_DEC, VALS(its_TrafficConditionSubCauseCode_vals), 0,
-        "SubCauseCodeType", HFILL }},
 
     /*
      * DENM SSP
@@ -1077,12 +918,13 @@ void proto_register_its(void)
 
     expert_register_field_array(expert_its, ei, array_length(ei));
 
-    register_dissector("its", dissect_its_PDU, proto_its);
+    its_handle = register_dissector("its", dissect_its_PDU, proto_its);
 
     // Register subdissector table
     its_version_subdissector_table = register_dissector_table("its.version", "ITS version", proto_its, FT_UINT8, BASE_DEC);
     its_msgid_subdissector_table = register_dissector_table("its.msg_id", "ITS message id", proto_its, FT_UINT32, BASE_DEC);
     regionid_subdissector_table = register_dissector_table("dsrc.regionid", "DSRC RegionId", proto_its, FT_UINT32, BASE_DEC);
+    cpmcontainer_subdissector_table = register_dissector_table("cpm.container", "CPM Containers id", proto_its, FT_UINT32, BASE_DEC);
     cam_pt_activation_table = register_dissector_table("cam.ptat", "CAM PtActivationType", proto_its, FT_UINT32, BASE_DEC);
 
     proto_its_denm = proto_register_protocol_in_name_only("ITS message - DENM", "DENM", "its.message.denm", proto_its, FT_BYTES);
@@ -1103,6 +945,8 @@ void proto_register_its(void)
     proto_its_evcsn = proto_register_protocol_in_name_only("ITS message - EVCSN", "EVCSN", "its.message.evcsn", proto_its, FT_BYTES);
     proto_its_tistpg = proto_register_protocol_in_name_only("ITS message - TISTPG", "TISTPG", "its.message.tistpg", proto_its, FT_BYTES);
     proto_its_cpm = proto_register_protocol_in_name_only("ITS message - CPM", "CPM", "its.message.cpm", proto_its, FT_BYTES);
+    proto_its_vam = proto_register_protocol_in_name_only("ITS message - VAM", "VAM", "its.message.vam", proto_its, FT_BYTES);
+    proto_its_imzm = proto_register_protocol_in_name_only("ITS message - IMZM", "IMZM", "its.message.imzm", proto_its, FT_BYTES);
 
     proto_addgrpc = proto_register_protocol_in_name_only("DSRC Addition Grp C (EU)", "ADDGRPC", "dsrc.addgrpc", proto_its, FT_BYTES);
 
@@ -1133,28 +977,28 @@ void proto_register_its(void)
 #define ITS_RTCMEM_PROT_VERv1 1
 #define ITS_RTCMEM_PROT_VER 2
 #define ITS_TIS_TPG_PROT_VER 1
-#define ITS_CPM_PROT_VER 1
+#define ITS_CPM_PROT_VER 2
+#define ITS_VAM_PROT_VER 2
+#define ITS_IMZM_PROT_VER 2
 
 void proto_reg_handoff_its(void)
 {
     const char *subdissector[BTP_SUBDISS_SZ] = { "btpa.port", "btpb.port" };
     const guint16 ports[BTP_PORTS_SZ] = { ITS_WKP_DEN, ITS_WKP_CA, ITS_WKP_EVCSN, ITS_WKP_CHARGING, ITS_WKP_IVI, ITS_WKP_TPG, ITS_WKP_TLC_SSEM, ITS_WKP_GPC, ITS_WKP_TLC_SREM, ITS_WKP_RLT, ITS_WKP_TLM, ITS_WKP_CPS };
     int sdIdx, pIdx;
-    dissector_handle_t its_handle_;
 
     // Register well known ports to btp subdissector table (BTP A and B)
-    its_handle_ = create_dissector_handle(dissect_its_PDU, proto_its);
     for (sdIdx=0; sdIdx < BTP_SUBDISS_SZ; sdIdx++) {
         for (pIdx=0; pIdx < BTP_PORTS_SZ; pIdx++) {
-            dissector_add_uint(subdissector[sdIdx], ports[pIdx], its_handle_);
+            dissector_add_uint(subdissector[sdIdx], ports[pIdx], its_handle);
         }
     }
 
     // Enable decode as for its pdu's send via udp
-    dissector_add_for_decode_as("udp.port", its_handle_);
+    dissector_add_for_decode_as("udp.port", its_handle);
 
-    dissector_add_uint("its.msg_id", (ITS_DENM_PROT_VER << 16) + ITS_DENM,          create_dissector_handle( dissect_denm_DecentralizedEnvironmentalNotificationMessage_PDU, proto_its_denm ));
-    dissector_add_uint("its.msg_id", (ITS_DENM_PROT_VERv1 << 16) + ITS_DENM,        create_dissector_handle( dissect_denmv1_DecentralizedEnvironmentalNotificationMessageV1_PDU, proto_its_denmv1 ));
+    dissector_add_uint("its.msg_id", (ITS_DENM_PROT_VER << 16) + ITS_DENM,          create_dissector_handle(dissect_denm_DenmPayload_PDU, proto_its_denm ));
+    dissector_add_uint("its.msg_id", (ITS_DENM_PROT_VERv1 << 16) + ITS_DENM,        create_dissector_handle(dissect_denmv1_DecentralizedEnvironmentalNotificationMessageV1_PDU, proto_its_denmv1 ));
     dissector_add_uint("its.msg_id", (ITS_CAM_PROT_VER << 16) + ITS_CAM,            create_dissector_handle( dissect_cam_CoopAwareness_PDU, proto_its_cam ));
     dissector_add_uint("its.msg_id", (ITS_CAM_PROT_VERv1 << 16) + ITS_CAM,          create_dissector_handle( dissect_camv1_CoopAwarenessV1_PDU, proto_its_camv1));
     dissector_add_uint("its.msg_id", (ITS_SPATEM_PROT_VERv1 << 16) + ITS_SPATEM,    create_dissector_handle( dissect_dsrc_SPAT_PDU, proto_its_spatemv1 ));
@@ -1171,6 +1015,8 @@ void proto_reg_handoff_its(void)
     dissector_add_uint("its.msg_id", ITS_EVCSN,                                     create_dissector_handle( dissect_evcsn_EVChargingSpotNotificationPOIMessage_PDU, proto_its_evcsn ));
     dissector_add_uint("its.msg_id", (ITS_TIS_TPG_PROT_VER << 16) + ITS_TISTPGTRANSACTION, create_dissector_handle( dissect_tistpg_TisTpgTransaction_PDU, proto_its_tistpg ));
     dissector_add_uint("its.msg_id", (ITS_CPM_PROT_VER << 16) + ITS_CPM,            create_dissector_handle(dissect_cpm_CollectivePerceptionMessage_PDU, proto_its_cpm));
+    dissector_add_uint("its.msg_id", (ITS_IMZM_PROT_VER << 16) + ITS_IMZM,          create_dissector_handle(dissect_imzm_InterferenceManagementZoneMessage_PDU, proto_its_imzm));
+    dissector_add_uint("its.msg_id", (ITS_VAM_PROT_VER << 16) + ITS_VAM,            create_dissector_handle(dissect_vam_VruAwareness_PDU, proto_its_vam));
 
     /* Missing definitions: ITS_POI, ITS_SAEM */
 
@@ -1191,6 +1037,12 @@ void proto_reg_handoff_its(void)
     dissector_add_uint("geonw.ssp", psid_den_basic_services, create_dissector_handle(dissect_denmssp_pdu, proto_its_denm));
     dissector_add_uint("geonw.ssp", psid_ca_basic_services,  create_dissector_handle(dissect_camssp_pdu, proto_its_cam));
 
+    dissector_add_uint("cpm.container", 1, create_dissector_handle(dissect_cpm_OriginatingVehicleContainer_PDU, proto_its_cpm));
+    dissector_add_uint("cpm.container", 2, create_dissector_handle(dissect_cpm_OriginatingRsuContainer_PDU, proto_its_cpm));
+    dissector_add_uint("cpm.container", 3, create_dissector_handle(dissect_cpm_SensorInformationContainer_PDU, proto_its_cpm));
+    dissector_add_uint("cpm.container", 4, create_dissector_handle(dissect_cpm_PerceptionRegionContainer_PDU, proto_its_cpm));
+    dissector_add_uint("cpm.container", 5, create_dissector_handle(dissect_cpm_PerceivedObjectContainer_PDU, proto_its_cpm));
+
     its_tap = register_tap("its");
 }
 
