@@ -8,6 +8,7 @@
  */
 
 #include <config.h>
+#define WS_LOG_DOMAIN  LOG_DOMAIN_MAIN
 
 #include <glib.h>
 
@@ -18,13 +19,14 @@
 #include <tchar.h>
 #include <wchar.h>
 #include <shellapi.h>
-#include "ui/win32/console_win32.h"
+#include <wsutil/console_win32.h>
 #endif
 
-#include <ui/clopts_common.h>
-#include <ui/cmdarg_err.h>
-#include <ui/exit_codes.h>
+#include <ws_exit_codes.h>
+#include <wsutil/clopts_common.h>
+#include <wsutil/cmdarg_err.h>
 #include <ui/urls.h>
+#include <wsutil/time_util.h>
 #include <wsutil/filesystem.h>
 #include <wsutil/privileges.h>
 #include <wsutil/socket.h>
@@ -35,7 +37,7 @@
 #include <wsutil/report_message.h>
 #include <wsutil/please_report_bug.h>
 #include <wsutil/unicode-utils.h>
-#include <ui/version_info.h>
+#include <wsutil/version_info.h>
 
 #include <epan/addr_resolv.h>
 #include <epan/ex-opt.h>
@@ -63,7 +65,6 @@
 #include "epan/srt_table.h"
 
 #include "ui/alert_box.h"
-#include "ui/console.h"
 #include "ui/iface_lists.h"
 #include "ui/language.h"
 #include "ui/persfilepath_opt.h"
@@ -227,11 +228,6 @@ gather_wireshark_qt_compiled_info(feature_list l)
     without_feature(l, "AirPcap");
 #endif
 #endif /* _WIN32 */
-#ifdef HAVE_SPEEXDSP
-    with_feature(l, "SpeexDSP (using system library)");
-#else
-    with_feature(l, "SpeexDSP (using bundled resampler)");
-#endif
 
 #ifdef HAVE_MINIZIP
     with_feature(l, "Minizip");
@@ -271,20 +267,41 @@ gather_wireshark_runtime_info(feature_list l)
         } else {
             without_feature(l, "HiDPI");
         }
+        QString session = qEnvironmentVariable("XDG_SESSION_TYPE");
+        if (!session.isEmpty()) {
+            if (session == "wayland") {
+                with_feature(l, "Wayland");
+            } else if (session == "x11") {
+                with_feature(l, "Xorg");
+            } else {
+                with_feature(l, "XDG_SESSION_TYPE=%s", qUtf8Printable(session));
+            }
+        }
+        QString platform = qApp->platformName();
+        if (!platform.isEmpty()) {
+            with_feature(l, "QPA plugin \"%s\"", qUtf8Printable(platform));
+        }
     }
 }
 
 static void
-qt_log_message_handler(QtMsgType type, const QMessageLogContext &, const QString &msg)
+qt_log_message_handler(QtMsgType type, const QMessageLogContext &context, const QString &msg)
 {
-    enum ws_log_level log_level = LOG_LEVEL_DEBUG;
+    enum ws_log_level log_level = LOG_LEVEL_NONE;
+
+    // QMessageLogContext may contain null/zero for release builds.
+    const char *file = context.file;
+    int line = context.line > 0 ? context.line : -1;
+    const char *func = context.function;
 
     switch (type) {
     case QtInfoMsg:
         log_level = LOG_LEVEL_INFO;
+        // Omit the file/line/function for this level.
+        file = nullptr;
+        line = -1;
+        func = nullptr;
         break;
-    // We want qDebug() messages to show up at our default log level.
-    case QtDebugMsg:
     case QtWarningMsg:
         log_level = LOG_LEVEL_WARNING;
         break;
@@ -294,10 +311,24 @@ qt_log_message_handler(QtMsgType type, const QMessageLogContext &, const QString
     case QtFatalMsg:
         log_level = LOG_LEVEL_ERROR;
         break;
-    default:
+    // We want qDebug() messages to show up always for temporary print-outs.
+    case QtDebugMsg:
+        log_level = LOG_LEVEL_ECHO;
         break;
     }
-    ws_log(LOG_DOMAIN_QTUI, log_level, "%s", qUtf8Printable(msg));
+
+    // Qt gives the full method declaration as the function. Our convention
+    // (following the C/C++ standards) is to display only the function name.
+    // Hack the name into the message as a workaround to avoid formatting
+    // issues.
+    if (func != nullptr) {
+        ws_log_full(LOG_DOMAIN_QTUI, log_level, file, line, nullptr,
+                        "%s -- %s", func, qUtf8Printable(msg));
+    }
+    else {
+        ws_log_full(LOG_DOMAIN_QTUI, log_level, file, line, nullptr,
+                        "%s", qUtf8Printable(msg));
+    }
 }
 
 #ifdef HAVE_LIBPCAP
@@ -326,7 +357,7 @@ check_and_warn_user_startup()
 }
 #endif
 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(__MINGW32__)
 // Try to avoid library search path collisions. QCoreApplication will
 // search QT_INSTALL_PREFIX/plugins for platform DLLs before searching
 // the application directory. If
@@ -344,6 +375,12 @@ check_and_warn_user_startup()
 // same path on the build machine. At any rate, loading DLLs from paths
 // you don't control is ill-advised. We work around this by removing every
 // path except our application directory.
+//
+// NOTE: This does not apply to MinGW-w64 using MSYS2. In that case we use
+// the system's Qt plugins with the default search paths.
+//
+// NOTE 2: When using MinGW-w64 without MSYS2 we also search for the system DLLS,
+// because our installer might not have deployed them.
 
 static inline void
 win32_reset_library_path(void)
@@ -406,6 +443,23 @@ macos_enable_layer_backing(void)
 }
 #endif
 
+#ifdef HAVE_LIBPCAP
+static GList *
+capture_opts_get_interface_list(int *err, char **err_str)
+{
+    /*
+     * XXX - should this pass an update callback?
+     * We already have a window up by the time we start parsing
+     * the majority of the command-line arguments, because
+     * we need to do a bunch of initialization work before
+     * parsing those arguments, and we want to let the user
+     * know that we're doing that initialization, given that
+     * it can take a while.
+     */
+    return capture_interface_list(err, err_str, NULL);
+}
+#endif
+
 /* And now our feature presentation... [ fade to music ] */
 int main(int argc, char *qt_argv[])
 {
@@ -430,6 +484,7 @@ int main(int argc, char *qt_argv[])
 #endif
 #endif
     gchar               *err_msg = NULL;
+    df_error_t          *df_err = NULL;
 
     QString              dfilter, read_filter;
 #ifdef HAVE_LIBPCAP
@@ -485,7 +540,7 @@ int main(int argc, char *qt_argv[])
     cmdarg_err_init(wireshark_cmdarg_err, wireshark_cmdarg_err_cont);
 
     /* Initialize log handler early so we can have proper logging during startup. */
-    ws_log_init_with_writer("wireshark", console_log_writer, vcmdarg_err);
+    ws_log_init("wireshark", vcmdarg_err);
     /* For backward compatibility with GLib logging and Wireshark 3.4. */
     ws_log_console_writer_set_use_stdout(TRUE);
 
@@ -514,6 +569,8 @@ int main(int argc, char *qt_argv[])
     setlocale(LC_ALL, "");
 #endif
 
+    ws_tzset();
+
 #ifdef _WIN32
     //
     // On Windows, QCoreApplication has its own WinMain(), which gets the
@@ -537,7 +594,8 @@ int main(int argc, char *qt_argv[])
 #endif /* _WIN32 */
 
     /* Early logging command-line initialization. */
-    ws_log_parse_args(&argc, argv, vcmdarg_err, INVALID_OPTION);
+    ws_log_parse_args(&argc, argv, vcmdarg_err, WS_EXIT_INVALID_OPTION);
+    ws_noisy("Finished log init and parsing command line log arguments");
 
     /*
      * Get credential information for later use, and drop privileges
@@ -607,6 +665,8 @@ int main(int argc, char *qt_argv[])
     ws_init_version_info("Wireshark", gather_wireshark_qt_compiled_info,
                          gather_wireshark_runtime_info);
 
+    init_report_message("Wireshark", &wireshark_report_routines);
+
     /* Create the user profiles directory */
     if (create_profiles_dir(&rf_path) == -1) {
         simple_dialog(ESD_TYPE_WARN, ESD_BTN_OK,
@@ -629,7 +689,7 @@ int main(int argc, char *qt_argv[])
 
     commandline_early_options(argc, argv);
 
-#ifdef _WIN32
+#if defined(_WIN32) && !defined(__MINGW32__)
     win32_reset_library_path();
 #endif
 
@@ -643,13 +703,21 @@ int main(int argc, char *qt_argv[])
     // https://doc.qt.io/qt-5/highdpi.html
     // https://bugreports.qt.io/browse/QTBUG-53022 - The device pixel ratio is pretty much bogus on Windows.
     // https://bugreports.qt.io/browse/QTBUG-55510 - Windows have wrong size
-#if defined(Q_OS_WIN)
+    //
+    // Deprecated in Qt6.
+    //    warning: 'Qt::AA_EnableHighDpiScaling' is deprecated: High-DPI scaling is always enabled.
+    //    This attribute no longer has any effect.
+#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     QApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
 #endif
 
     /* Create The Wireshark app */
     WiresharkApplication ws_app(argc, qt_argv);
 
+    // Default value is 400ms = "quickly typing" when searching in Preferences->Protocols
+    // 1000ms allows a more "hunt/peck" typing speed. 2000ms tested - too long.
+    QApplication::setKeyboardInputInterval(1000);
+
     /* initialize the funnel mini-api */
     // xxx qtshark
     //initialize_funnel_ops();
@@ -665,7 +733,7 @@ int main(int argc, char *qt_argv[])
         cmdarg_err("%s", err_msg);
         g_free(err_msg);
         cmdarg_err_cont("%s", please_report_bug());
-        ret_val = INIT_FAILED;
+        ret_val = WS_EXIT_INIT_FAILED;
         goto clean_exit;
     }
 
@@ -699,13 +767,15 @@ int main(int argc, char *qt_argv[])
     main_w->connect(&ws_app, &WiresharkApplication::openCaptureOptions,
             main_w, &WiresharkMainWindow::showCaptureOptionsDialog);
 
-    /* Init the "Open file" dialog directory */
-    /* (do this after the path settings are processed) */
+    /*
+     * If we have a saved "last directory in which a file was opened"
+     * in the recent file, set it as the one for the app.
+     *
+     * (do this after the path settings are processed)
+     */
     if (recent.gui_fileopen_remembered_dir &&
         test_for_directory(recent.gui_fileopen_remembered_dir) == EISDIR) {
-      wsApp->setLastOpenDir(recent.gui_fileopen_remembered_dir);
-    } else {
-      wsApp->setLastOpenDir(get_persdatafile_dir());
+      set_last_open_dir(recent.gui_fileopen_remembered_dir);
     }
 
 #ifdef DEBUG_STARTUP_TIME
@@ -715,11 +785,9 @@ int main(int argc, char *qt_argv[])
 #ifdef HAVE_LIBPCAP
     /* Set the initial values in the capture options. This might be overwritten
        by preference settings and then again by the command line parameters. */
-    capture_opts_init(&global_capture_opts);
+    capture_opts_init(&global_capture_opts, capture_opts_get_interface_list);
 #endif
 
-    init_report_message("Wireshark", &wireshark_report_routines);
-
     /*
      * Libwiretap must be initialized before libwireshark is, so that
      * dissection-time handlers for file-type-dependent blocks can
@@ -737,7 +805,7 @@ int main(int argc, char *qt_argv[])
        case any dissectors register preferences. */
     if (!epan_init(splash_update, NULL, TRUE)) {
         SimpleDialog::displayQueuedMessages(main_w);
-        ret_val = INIT_FAILED;
+        ret_val = WS_EXIT_INIT_FAILED;
         goto clean_exit;
     }
 #ifdef DEBUG_STARTUP_TIME
@@ -861,7 +929,7 @@ int main(int argc, char *qt_argv[])
                 cmdarg_err("%s%s%s", err_str, err_str_secondary ? "\n" : "", err_str_secondary ? err_str_secondary : "");
                 g_free(err_str);
                 g_free(err_str_secondary);
-                ret_val = INVALID_CAPABILITY;
+                ret_val = WS_EXIT_INVALID_CAPABILITY;
                 break;
             }
             ret_val = capture_opts_print_if_capabilities(caps, interface_opts,
@@ -887,6 +955,7 @@ int main(int argc, char *qt_argv[])
 #ifdef DEBUG_STARTUP_TIME
     ws_log(LOG_DOMAIN_MAIN, LOG_LEVEL_INFO, "Calling prefs_apply_all, elapsed time %" PRIu64 " us \n", g_get_monotonic_time() - start_time);
 #endif
+    splash_update(RA_PREFERENCES_APPLY, NULL, NULL);
     prefs_apply_all();
     prefs_to_capture_opts();
     wsApp->emitAppSignal(WiresharkApplication::PreferencesChanged);
@@ -912,7 +981,7 @@ int main(int argc, char *qt_argv[])
      * command-line options.
      */
     if (!setup_enabled_and_disabled_protocols()) {
-        ret_val = INVALID_OPTION;
+        ret_val = WS_EXIT_INVALID_OPTION;
         goto clean_exit;
     }
 
@@ -920,7 +989,7 @@ int main(int argc, char *qt_argv[])
     wsApp->emitAppSignal(WiresharkApplication::ColumnsChanged); // We read "recent" widths above.
     wsApp->emitAppSignal(WiresharkApplication::RecentPreferencesRead); // Must be emitted after PreferencesChanged.
 
-    wsApp->setMonospaceFont(prefs.gui_qt_font_name);
+    wsApp->setMonospaceFont(prefs.gui_font_name);
 
     /* For update of WindowTitle (When use gui.window_title preference) */
     main_w->setWSWindowTitle();
@@ -930,6 +999,14 @@ int main(int argc, char *qt_argv[])
         g_free(err_msg);
     }
 
+    /* allSystemsGo() emits appInitialized(), which signals the WelcomePage to
+     * delete the splash overlay. However, it doesn't get redrawn until
+     * processEvents() is called. If we're opening a capture file that happens
+     * either when we finish reading the file or when the progress bar appears.
+     * It's probably better to leave the splash overlay up until that happens
+     * rather than showing the user the welcome page, so we don't call
+     * processEvents() here.
+     */
     wsApp->allSystemsGo();
     ws_log(LOG_DOMAIN_MAIN, LOG_LEVEL_INFO, "Wireshark is up and ready to go, elapsed time %.3fs", (float) (g_get_monotonic_time() - start_time) / 1000000);
     SimpleDialog::displayQueuedMessages(main_w);
@@ -956,13 +1033,13 @@ int main(int argc, char *qt_argv[])
             } else if (global_commandline_info.jfilter != NULL) {
                 dfilter_t *jump_to_filter = NULL;
                 /* try to compile given filter */
-                if (!dfilter_compile(global_commandline_info.jfilter, &jump_to_filter, &err_msg)) {
+                if (!dfilter_compile(global_commandline_info.jfilter, &jump_to_filter, &df_err)) {
                     // Similar code in MainWindow::mergeCaptureFile().
                     QMessageBox::warning(main_w, QObject::tr("Invalid Display Filter"),
                                          QObject::tr("The filter expression %1 isn't a valid display filter. (%2).")
-                                                 .arg(global_commandline_info.jfilter, err_msg),
+                                                 .arg(global_commandline_info.jfilter, df_err->msg),
                                          QMessageBox::Ok);
-                    g_free(err_msg);
+                    df_error_free(&df_err);
                 } else {
                     /* Filter ok, jump to the first packet matching the filter
                        conditions. Default search direction is forward, but if
